import { PlusOutlined } from "@ant-design/icons";
import {
  Button,
  Checkbox,
  Form,
  Grid,
  Popconfirm,
  Space,
  notification,
} from "antd";
import MyModal, {
  MyModalCloseCreateButtonFooter,
  MyModalCloseSaveButtonFooter,
} from "../../../Components/MyModal";
import { useEffect, useState } from "react";
import {
  myFetch,
  EncodeStringToBase64,
  showInputsInvalidNotification,
  showUnkownErrorNotification,
} from "../../../utils";
import { useTranslation } from "react-i18next";
import MyTable from "../../../Components/MyTable";
import {
  MyCalendarMaxFutureBookingDaysFormInput,
  MyCalendarMinEarliestBookingTimeFormInput,
  MyEmailFormInput,
  MyPasswordFormInput,
  MyUsernameFormInput,
} from "../../../Components/MyFormInputs";
import { Link, useParams } from "react-router-dom";
import CountUp from "react-countup";
import { useAppContext } from "../../../Contexts/AppContext";

const { useBreakpoint } = Grid;

export default function StoreEmployees() {
  const { t } = useTranslation();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  const appContext = useAppContext();

  const screenBreakpoint = useBreakpoint();
  const { storeId } = useParams();

  const [requestData, setRequestData] = useState({
    storeSettings: {},
    employees: [],
  });
  const [isRequesting, setIsRequesting] = useState(false);

  const [addEditEmployeeModalOptions, setAddEditEmployeeModalOptions] =
    useState({
      mode: "add",
      isOpen: false,
      selectedEmployee: null,
    });

  const getTableColumns = () => {
    return [
      {
        title: t("common.email"),
        dataIndex: "email",
        key: "email",
      },
      {
        title: t("common.username"),
        dataIndex: "username",
        key: "username",
      },
      {
        title: t("common.calendarMaxFutureBookingDays"),
        dataIndex: "calendarMaxFutureBookingDays",
        key: "calendarMaxFutureBookingDays",
      },
      {
        title: t("common.calendarMinEarliestBookingTime"),
        dataIndex: "calendarMinEarliestBookingTime",
        key: "calendarMinEarliestBookingTime",
      },
      {
        title: t("common.action"),
        dataIndex: "action",
        key: "actions",
        render: (_, record) => (
          <Space>
            <Link
              to="#"
              onClick={() => {
                setAddEditEmployeeModalOptions({
                  ...addEditEmployeeModalOptions,
                  mode: "edit",
                  isOpen: true,
                  selectedEmployee: record,
                });
              }}
            >
              {t("common.button.edit")}
            </Link>
            <Popconfirm
              title={t("employees.popConfirmDeleteEmployee.title")}
              description={t("employees.popConfirmDeleteEmployee.description")}
              okText={t("common.button.delete")}
              onConfirm={() => {
                setIsRequesting(true);

                myFetch({
                  url: "/users",
                  method: "DELETE",
                  body: { userId: record.key },
                  notificationApi: notificationApi,
                  t: t,
                })
                  .then(() => fetchEmployees())
                  .catch(() => setIsRequesting(false));
              }}
            >
              <Link to="#">{t("common.button.delete")}</Link>
            </Popconfirm>
          </Space>
        ),
      },
    ];
  };

  const getTableItems = () => {
    return requestData.employees.map((employee) => {
      return {
        key: employee.user_id,
        email: employee.email,
        username: employee.username,
        calendarMaxFutureBookingDays:
          employee.calendar_max_future_booking_days ||
          requestData.storeSettings.calendar_max_future_booking_days,
        calendarMinEarliestBookingTime:
          employee.calendar_min_earliest_booking_time ||
          requestData.storeSettings.calendar_min_earliest_booking_time,
      };
    });
  };

  const fetchEmployees = () => {
    setIsRequesting(true);

    myFetch({
      url: `/users/${storeId}`,
      method: "GET",
      notificationApi: notification,
      t: t,
    })
      .then((data) => {
        setIsRequesting(false);
        setRequestData(data);
      })
      .catch(() => {});
  };

  useEffect(() => fetchEmployees(), []);

  return (
    <>
      {notificationContextHolder}

      <div
        style={{
          display: "flex",
          justifyContent: "space-between",
          flexDirection: screenBreakpoint.xs ? "column" : "row",
        }}
      >
        <h1>
          {t("employees.pageTitle")}

          {" ("}
          <CountUp end={requestData.employees.length} />
          {")"}
        </h1>

        <ModalAddEditEmployee
          storeSettings={requestData.storeSettings}
          modalOptions={addEditEmployeeModalOptions}
          setModalOptions={setAddEditEmployeeModalOptions}
          fetchEmployees={fetchEmployees}
          disabled={
            appContext.paymentPlanSettings.maxEmployees <=
            requestData.employees.length
          }
        />
      </div>

      <MyTable
        props={{
          loading: isRequesting,
          columns: getTableColumns(),
          dataSource: getTableItems(),
        }}
      />
    </>
  );
}

function ModalAddEditEmployee({
  storeSettings,
  modalOptions,
  setModalOptions,
  fetchEmployees,
  disabled,
}) {
  const { t, i18n } = useTranslation();
  const [notificationApi, notificationContextHolder] =
    notification.useNotification();
  const { storeId } = useParams();
  const screenBreakpoint = useBreakpoint();
  const [form] = Form.useForm();

  const checkboxSetPasswordOnLogging = Form.useWatch(
    "checkboxSetPasswordOnLogging",
    form
  );

  const [isRequesting, setIsRequesting] = useState(false);

  const handleModalClose = () => {
    setModalOptions({
      ...modalOptions,
      isOpen: false,
    });

    form.resetFields();
  };

  useEffect(() => {
    if (!modalOptions.isOpen) return;

    if (modalOptions.mode === "edit") {
      form.setFieldsValue({
        username: modalOptions.selectedEmployee.username,
        email: modalOptions.selectedEmployee.email,
        calendarMaxFutureBookingDays:
          modalOptions.selectedEmployee.calendarMaxFutureBookingDays ||
          storeSettings.calendar_max_future_booking_days,
        calendarMinEarliestBookingTime:
          modalOptions.selectedEmployee.calendarMinEarliestBookingTime ||
          storeSettings.calendar_min_earliest_booking_time,
      });
    } else {
      // set default values
      form.setFieldsValue({
        calendarMaxFutureBookingDays:
          storeSettings.calendar_max_future_booking_days,
        calendarMinEarliestBookingTime:
          storeSettings.calendar_min_earliest_booking_time,
      });
    }
  }, [modalOptions.isOpen]);

  return (
    <>
      {notificationContextHolder}

      <Button
        type="primary"
        block={screenBreakpoint.xs}
        icon={<PlusOutlined />}
        onClick={() =>
          setModalOptions({ ...modalOptions, mode: "add", isOpen: true })
        }
        disabled={disabled}
      >
        {t("employees.addEmployee")}
      </Button>

      <MyModal
        title={
          modalOptions.mode === "add"
            ? t("employees.addEmployee")
            : t("employees.editEmployee")
        }
        isOpen={modalOptions.isOpen}
        onCancel={handleModalClose}
        footer={
          modalOptions.mode === "add" ? (
            <MyModalCloseCreateButtonFooter
              onCancel={handleModalClose}
              isCreateButtonLoading={isRequesting}
              onCreate={() => {
                form
                  .validateFields()
                  .then((values) => {
                    setIsRequesting(true);

                    let body = {
                      storeId: storeId,
                      username: values.username,
                      email: values.email,
                      language: i18n.language,
                      passwordSetOnInitLogging:
                        values.checkboxSetPasswordOnLogging,
                    };

                    if (
                      !values.checkboxSetPasswordOnLogging &&
                      values.password
                    ) {
                      body.password = EncodeStringToBase64(values.password);
                    }

                    if (
                      values.calendarMaxFutureBookingDays !==
                      storeSettings.calendar_max_future_booking_days
                    ) {
                      body.calendarMaxFutureBookingDays =
                        values.calendarMaxFutureBookingDays;
                    }

                    if (
                      values.calendarMinEarliestBookingTime !==
                      storeSettings.calendar_min_earliest_booking_time
                    ) {
                      body.calendarMinEarliestBookingTime =
                        values.calendarMinEarliestBookingTime;
                    }

                    myFetch({
                      url: "/users",
                      method: "POST",
                      body: body,
                      notificationApi: notification,
                      t: t,
                    })
                      .then(() => {
                        setIsRequesting(false);
                        handleModalClose();

                        fetchEmployees();
                      })
                      .catch(() => {
                        showUnkownErrorNotification(notificationApi, t);
                        setIsRequesting(false);
                      });
                  })
                  .catch(() =>
                    showInputsInvalidNotification(notificationApi, t)
                  );
              }}
            />
          ) : (
            <MyModalCloseSaveButtonFooter
              onCancel={handleModalClose}
              isSaveButtonLoading={isRequesting}
              onSave={() => {
                // only validate if something has changed
                const formUsername = form.getFieldValue("username");
                const formEmail = form.getFieldValue("email");
                const formCalendarMaxFutureBookingDays = form.getFieldValue(
                  "calendarMaxFutureBookingDays"
                );
                const formCalendarMinEarliestBookingTime = form.getFieldValue(
                  "calendarMinEarliestBookingTime"
                );

                if (
                  formUsername === modalOptions.selectedEmployee.username &&
                  formEmail === modalOptions.selectedEmployee.email &&
                  formCalendarMaxFutureBookingDays ===
                    modalOptions.selectedEmployee
                      .calendarMaxFutureBookingDays &&
                  formCalendarMinEarliestBookingTime ===
                    modalOptions.selectedEmployee.calendarMinEarliestBookingTime
                ) {
                  handleModalClose();
                  return;
                }

                let validateFields = [];
                let body = {
                  userId: modalOptions.selectedEmployee.key,
                };

                if (formUsername !== modalOptions.selectedEmployee.username) {
                  validateFields.push("username");
                  body.username = formUsername;
                }

                if (formEmail !== modalOptions.selectedEmployee.email) {
                  validateFields.push("email");
                  body.email = formEmail;
                }

                if (
                  formCalendarMaxFutureBookingDays !==
                  modalOptions.selectedEmployee.calendarMaxFutureBookingDays
                ) {
                  validateFields.push("calendarMaxFutureBookingDays");
                  body.calendarMaxFutureBookingDays =
                    formCalendarMaxFutureBookingDays;
                }

                if (
                  formCalendarMinEarliestBookingTime !==
                  modalOptions.selectedEmployee.calendarMinEarliestBookingTime
                ) {
                  validateFields.push("calendarMinEarliestBookingTime");
                  body.calendarMinEarliestBookingTime =
                    formCalendarMinEarliestBookingTime;
                }

                form
                  .validateFields(validateFields)
                  .then(() => {
                    setIsRequesting(true);

                    myFetch({
                      url: "/users/update",
                      method: "POST",
                      body: body,
                      notificationApi: notification,
                      t: t,
                    })
                      .then(() => {
                        setIsRequesting(false);
                        handleModalClose();

                        fetchEmployees();
                      })
                      .catch(() => setIsRequesting(false));
                  })
                  .catch(() =>
                    showInputsInvalidNotification(notificationApi, t)
                  );
              }}
            />
          )
        }
      >
        <Form
          form={form}
          layout="vertical"
          requiredMark={false}
          initialValues={{
            checkboxSetPasswordOnLogging: true,
          }}
        >
          <MyUsernameFormInput thirdPerson />

          <MyEmailFormInput inputNotRequired={true} hasFeedback thirdPerson />

          {modalOptions.mode === "add" && (
            <>
              <Form.Item
                name="checkboxSetPasswordOnLogging"
                valuePropName="checked"
                extra={t(
                  "employees.modalAddEmployee.checkboxSetPasswordOnLoggingInfo"
                )}
              >
                <Checkbox>
                  {t("employees.modalAddEmployee.checkboxSetPasswordOnLogging")}
                </Checkbox>
              </Form.Item>

              {!checkboxSetPasswordOnLogging && <MyPasswordFormInput />}
            </>
          )}

          <MyCalendarMaxFutureBookingDaysFormInput
            maxLength={storeSettings.calendar_max_future_booking_days}
          />

          <MyCalendarMinEarliestBookingTimeFormInput
            maxLength={storeSettings.calendar_min_earliest_booking_time}
          />
        </Form>
      </MyModal>
    </>
  );
}
